@@ -1,7 +1,8 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil -*-
-
-/* Blackjack - hand.cpp
- * Copyright (C) 2003 William Jon McCann <mccann@jhu.edu>
+/*
+ * Blackjack - hand.cpp
+ *
+ * Copyright (C) 2003-2005 William Jon McCann <mccann@jhu.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,17 +22,10 @@
 
 #include <config.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
-#include <sys/types.h>
-#include <string.h>
 #include <stdlib.h>
-#include <time.h>
-#include <dirent.h>
-#include <ctype.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "blackjack.h"
 #include "events.h"
@@ -42,13 +36,26 @@
 #include "hand.h"
 #include "chips.h"
 
-#include <iostream>
 using namespace std;
 
 guint deal_timeout_id = 0;
 guint finish_timeout_id = 0;
 guint hit_timeout_id = 0;
 
+static void bj_hand_finish_internal (void);
+
+gboolean
+bj_hand_events_pending (void)
+{
+        gboolean pending = FALSE;
+
+        pending = (deal_timeout_id != 0
+                   || hit_timeout_id != 0
+                   || finish_timeout_id != 0);
+
+        return pending;
+}
+
 void
 bj_hand_cancel ()
 {
@@ -66,8 +73,6 @@ bj_hand_cancel ()
                 g_source_remove (finish_timeout_id);
                 finish_timeout_id = 0;
         }
-
-        events_pending = false;
 }
 
 hslot_type
@@ -98,8 +103,8 @@ bj_hand_show_dealer_probabilities ()
                                                         dealer->cards[0].value());
 }
 
-gchar *
-bj_hand_get_best_option_string ()
+char *
+bj_hand_get_best_option_string (char **secondary_message)
 {
         gchar *option_str = NULL;
         gint bestOption;
@@ -113,14 +118,31 @@ bj_hand_get_best_option_string ()
 
         switch (bestOption) {
         case KEY_S : option_str = g_strdup (_("The best option is to stand"));
+                if (secondary_message)
+                        *secondary_message = g_strdup (_("To stand means to stop adding cards to your hand.  "
+                                                         "Do this by clicking on the dealer's cards or by selecting the option from the Control menu."));
                 break;
         case KEY_H : option_str = g_strdup (_("The best option is to hit"));
+                if (secondary_message)
+                        *secondary_message = g_strdup (_("To hit means to add another card to your hand.  "
+                                                         "Do this by clicking once on your cards or by selecting the option from the Control menu."));
                 break;
         case KEY_D : option_str = g_strdup (_("The best option is to double down"));
+                if (secondary_message)
+                        *secondary_message = g_strdup (_("To double down means to double the initial wager and receive exactly one more card.  "
+                                                         "Do this by clicking once on the chips at the bottom of the window or by selecting the option from the Control menu."));
                 break;
         case KEY_P : option_str = g_strdup (_("The best option is to split"));
+                if (secondary_message)
+                        *secondary_message = g_strdup (_("To split means to divide your current hand into two separate hands.  Do this by dragging one of your cards and dropping it off to the side or by selecting the option from the Control menu."));
                 break;
         case KEY_R : option_str = g_strdup (_("The best option is to surrender"));
+                if (secondary_message)
+                        *secondary_message = g_strdup (_("To surrender means to give up half your wager and not complete the hand.  "
+                                                         "Do this by selecting the option from the Control menu."));
+                break;
+        default:
+                break;
         }
 
         return option_str;
@@ -149,59 +171,59 @@ gboolean
 bj_hand_can_be_doubled ()
 {
         if (! bj_game_is_active ())
-                return false;
+                return FALSE;
 
         // must be able to hit to double down
         if (! bj_hand_can_be_hit ())
-                return false;
+                return FALSE;
 
         if ((numHands == 1 && rules->getDoubleDown (*player))
             || (numHands > 1 && rules->getDoubleAfterSplit (*player)))
-                return true;
+                return TRUE;
         else 
-                return false;
+                return FALSE;
 }
 
 gboolean
 bj_hand_can_be_hit ()
 {
         if (! bj_game_is_active ())
-                return false;
+                return FALSE;
 
         if (numHands == 1 || player->cards[0].value() != 1)
-                return true;
+                return TRUE;
         else
-                return false;
+                return FALSE;
 }
 
 gboolean
 bj_hand_can_be_surrendered ()
 {
         if (! bj_game_is_active ())
-                return false;
+                return FALSE;
 
         if (rules->getLateSurrender () 
             && player->getCards () == 2
             && numHands == 1)
-                return true;
+                return TRUE;
         else
-                return false;
+                return FALSE;
 }
 
 gboolean
 bj_hand_can_be_split ()
 {
         if (! bj_game_is_active ())
-                return false;
+                return FALSE;
 
         // Check if player can split a pair.
 
         int card = player->cards[0].value ();
         if (player->getCards () == 2 && card == player->cards[1].value ()
             && numHands < rules->getResplit (card))
-                return true;
+                return TRUE;
         else
-                return false;
+                return FALSE;
 }
 
 void
@@ -211,33 +233,32 @@ bj_deal_card_to_player ()
 }
 
 void
-bj_deal_card_to_dealer (gboolean faceup=true)
+bj_deal_card_to_dealer (gboolean faceup=TRUE)
 {
         dealer->deal (shoe->deal (), faceup);
 }
 
 void
-bj_deal_card_to_dealer_distribution (gboolean faceup=true)
+bj_deal_card_to_dealer_distribution (gboolean faceup=TRUE)
 {
         distribution->deal (dealer->deal (shoe->deal ()));
 }
 
-
 static gboolean
 bj_hand_new5 (gpointer data)
 {
 
         player->showCount ();
-        allSettled = false;
+        allSettled = FALSE;
 
         // Ask for insurance if the up card is an ace.
 
-        bool insurance = false;
-        if (dealer->cards[0].value () == 1 || dealer->cards[0].value () == 10) {
+        bool insurance = FALSE;
+        if (!bj_get_never_insurance () && (dealer->cards[0].value () == 1 || dealer->cards[0].value () == 10)) {
                 if (bj_get_show_probabilities ()) {
                         dealerProbabilities->showProbabilities (distribution,
                                                                 dealer->cards[0].value (),
-                                                                false);
+                                                                FALSE);
                 }
                 if (dealer->cards[0].value () == 1) {
                         if (insurance = get_insurance_choice ())
@@ -248,14 +269,14 @@ bj_hand_new5 (gpointer data)
         // Check for dealer blackjack.
         
         if (dealer->getCards () == 2 && dealer->getCount () == 21) {
-                allSettled = true;
+                allSettled = TRUE;
                 if (insurance)
                         bj_adjust_balance (player->wager / 2 + (player->wager / 2) * 2);
             
                 if ( (player->getCards () == 2) && (player->getCount () == 21) )
                         bj_adjust_balance (player->wager);
-                bj_game_set_active (false);
-                bj_hand_finish ();
+                bj_game_set_active (FALSE);
+                bj_hand_finish_internal ();
                 deal_timeout_id = 0;
                 return FALSE;
         }
@@ -265,7 +286,7 @@ bj_hand_new5 (gpointer data)
 
         else {
                 if ( (player->getCards () == 2) && (player->getCount () == 21) ) {
-                        allSettled = true;
+                        allSettled = TRUE;
                         bj_adjust_balance (player->wager + player->wager * 3 / 2);
                 }
         }
@@ -275,12 +296,12 @@ bj_hand_new5 (gpointer data)
         if (!allSettled) {
                 dealerProbabilities->showProbabilities (distribution,
                                                         dealer->cards[0].value ());
-                allSettled = true;
-                bj_game_set_active (true);
+                allSettled = TRUE;
+                bj_game_set_active (TRUE);
         }
 
         if (player->getCount () == 21) {
-                bj_hand_finish ();
+                bj_hand_finish_internal ();
                 deal_timeout_id = 0;
                 return FALSE;
         }
@@ -289,9 +310,9 @@ bj_hand_new5 (gpointer data)
                         strategy->showOptions (player,
                                                dealer->cards[0].value (), 
                                                numHands);
-                events_pending = false;
         }
-        
+
+        bj_update_control_menu ();
         bj_draw_refresh_screen ();
         deal_timeout_id = 0;
         return FALSE;
@@ -300,7 +321,7 @@ bj_hand_new5 (gpointer data)
 static gboolean
 bj_hand_new4 (gpointer data)
 {
-        bj_deal_card_to_dealer (false);
+        bj_deal_card_to_dealer (FALSE);
         bj_draw_refresh_screen ();
 
         deal_timeout_id = g_timeout_add ((gint)bj_get_deal_delay (),
@@ -352,6 +373,8 @@ bj_hand_new1 (gpointer data)
 void
 bj_hand_new ()
 {
+        if (bj_hand_events_pending ())
+                return;
 
         // Reshuffle if necessary.
         if (shoe->numCards < 52) {
@@ -361,22 +384,38 @@ bj_hand_new ()
 
         player->wager = bj_get_wager ();
         bj_adjust_balance (-1 * player->wager);
-        bj_game_set_active (true);
+        bj_game_set_active (TRUE);
   
         player->showWager ();
         lastWager = player->wager;
 
-        events_pending = true;
         bj_hand_new1 (NULL);
 }
 
+static gint
+bj_count_live_hands ()
+{
+	// Live hands compete with the dealer hand.
+	gint live_hands = 0; 
+	player = (PlayerHand*) g_list_nth_data (playerHands,0);
+	while (player != NULL) {
+		if ((player->getCount () < 22 ) 
+		    && !((player->getCards () == 2) && (player->getCount () == 21)))
+			live_hands++;
+		player = player->nextHand;
+	}
+	return live_hands;
+}
+
 static gboolean
 bj_hand_finish1 (gpointer data)
 {
-        if (dealer->getCount () < 17 
-            || (rules->getHitSoft17 () 
-                && dealer->getCount () == 17 
-                && dealer->getSoft ())) {
+        if (!allSettled
+	    && bj_count_live_hands () 
+	    && (dealer->getCount () < 17 
+                || (rules->getHitSoft17 () 
+                    && dealer->getCount () == 17 
+                    && dealer->getSoft ()))) {
                 // Finish dealer hand.
                 bj_deal_card_to_dealer_distribution ();
                 bj_draw_refresh_screen ();
@@ -398,42 +437,58 @@ bj_hand_finish1 (gpointer data)
                                 player = player->nextHand;
                         }
                 }
-                events_pending = false;
                 bj_draw_refresh_screen ();
                 finish_timeout_id = 0;
                 return FALSE;
         }
 }
 
-void
-bj_hand_finish ()
+static void
+bj_hand_finish_internal (void)
 {
-        events_pending = true;
-        bj_game_set_active (false);
+        hcard_type card;
+
+        bj_game_set_active (FALSE);
         // Turn dealer hole card.
-        hcard_type card = (hcard_type) g_list_nth_data (dealer->hslot->cards, 1);
-        card->direction = UP;
+
+        card = (hcard_type) g_list_nth_data (dealer->hslot->cards, 1);
+        if (card)
+                card->direction = UP;
+
         dealer->showCount ();
+        bj_update_control_menu ();
         bj_draw_refresh_screen ();
 
         finish_timeout_id = g_timeout_add ((gint)bj_get_deal_delay (), 
                                            bj_hand_finish1, NULL);
 }
 
+void
+bj_hand_finish ()
+{
+        if (bj_hand_events_pending ())
+                return;
+
+        bj_hand_finish_internal ();
+}
+
 void
 bj_hand_stand ()
 {
+        if (bj_hand_events_pending ())
+                return;
+
         if (bj_game_is_active ()) {
-                allSettled = false;
+                allSettled = FALSE;
                 if ((player = player->nextHand) == NULL) {
-                        bj_hand_finish ();
+                        bj_hand_finish_internal ();
                 }
                 bj_hand_finish_play ();
         }
 }
 
-void
-bj_hand_hit ()
+static void
+bj_hand_hit_internal ()
 {
         if (bj_hand_can_be_hit ()) {
                 bj_deal_card_to_player ();
@@ -441,8 +496,8 @@ bj_hand_hit ()
                 player->showCount ();
                 if (player->getCount () >= 21)
                         if ((player = player->nextHand) == NULL) {
-                                allSettled = false;
-                                bj_hand_finish ();
+                                allSettled = FALSE;
+                                bj_hand_finish_internal ();
                                 return;
                         }
                 bj_hand_finish_play ();
@@ -452,8 +507,7 @@ bj_hand_hit ()
 static gboolean
 bj_hand_hit_delay_cb (gpointer data)
 {
-        events_pending = false;
-        bj_hand_hit ();
+        bj_hand_hit_internal ();
         hit_timeout_id = 0;
         return FALSE;
 }
@@ -461,7 +515,9 @@ bj_hand_hit_delay_cb (gpointer data)
 void
 bj_hand_hit_with_delay (void)
 {
-        events_pending = true;
+        if (bj_hand_events_pending ())
+                return;
+
         hit_timeout_id = g_timeout_add ((gint)bj_get_deal_delay (),
                                         bj_hand_hit_delay_cb, NULL);
 }
@@ -469,6 +525,9 @@ bj_hand_hit_with_delay (void)
 void
 bj_hand_double ()
 {
+        if (bj_hand_events_pending ())
+                return;
+
         if (bj_hand_can_be_doubled ()) {
                 bj_adjust_balance (-1 * player->wager);
                 player->wager *= 2;
@@ -478,7 +537,7 @@ bj_hand_double ()
                 
                 player->showCount ();
                 if (player->getCount () <= 21)
-                        allSettled = false;
+                        allSettled = FALSE;
                 if ((player = player->nextHand) == NULL)
                         bj_hand_finish ();
                 else
@@ -492,6 +551,9 @@ bj_hand_split ()
         hslot_type hslot;
         gint slot_start_x;
 
+        if (bj_hand_events_pending ())
+                return;
+
         if (bj_hand_can_be_split ()) {
                 // Put card value back into shoe
                 tempCard = player->cards[1];
@@ -557,16 +619,22 @@ bj_hand_split ()
 void
 bj_hand_surrender ()
 {
+        if (bj_hand_events_pending ())
+                return;
+
         if (bj_hand_can_be_surrendered ()) {
                 bj_adjust_balance (player->wager / 2);
                 if ((player = player->nextHand) == NULL)
-                        bj_hand_finish ();
+                        bj_hand_finish_internal ();
         }
 }
 
 void
 bj_hand_new_deal ()
 {
+        if (bj_hand_events_pending ())
+                return;
+
         if (! bj_game_is_active ()) {
                 bj_clear_table ();
                 bj_hand_new ();
@@ -582,18 +650,18 @@ bj_hand_finish_play ()
                 // Deal another card to a split hand if necessary.
                 // need to loop for the case where splits get blackjack
                 // FIXME - this is less than elegant (get rid of while loop)
-                check_splits = true;
+                check_splits = TRUE;
                 while (check_splits) {
                         if (player->getCards () == 1)
                                 bj_deal_card_to_player ();
                         else
-                                check_splits = false;
+                                check_splits = FALSE;
 
                         if (player->getCount () >= 21) 
                                 if (player->nextHand == NULL) {
-                                        check_splits = false;
-                                        allSettled = false;
-                                        bj_hand_finish ();
+                                        check_splits = FALSE;
+                                        allSettled = FALSE;
+                                        bj_hand_finish_internal ();
                                 }
                                 else
                                         player = player->nextHand;
@@ -602,5 +670,7 @@ bj_hand_finish_play ()
                         bj_hand_show_options ();
                 }
         }
+
+        bj_update_control_menu ();
         bj_draw_refresh_screen ();
 }
