@@ -12,6 +12,12 @@ import traceback
 import time
 import gettext
 
+try:
+    import pygtk
+    pygtk.require('2.0')
+except ImportError, err:
+    print ("PyGTK not found. Please make sure it is installed properly and referenced in your PYTHONPATH environment variable.")
+
 import gobject
 import gtk
 import gtk.glade
@@ -27,10 +33,12 @@ else:
 from glchess.defaults import *
 
 # Optionally use OpenGL support
+# gtk.gtkgl throws RuntimeError when there is no OpenGL support
+# It should really just throw ImportError
 try:
     import gtk.gtkgl
     import OpenGL
-except ImportError:
+except (ImportError, RuntimeError):
     haveGLSupport = False
 else:
     haveGLSupport = True
@@ -42,8 +50,8 @@ import glchess.config
 import glchess.ui
 import dialogs
 
-def loadGladeFile(name, root = None, domain = None):
-    return gtk.glade.XML(os.path.join(GLADE_DIR, name), root, domain = domain)
+def loadGladeFile(name, root = None):
+    return gtk.glade.XML(os.path.join(GLADE_DIR, name), root, domain = DOMAIN)
 
 class GtkViewArea(gtk.DrawingArea):
     """Custom widget to render an OpenGL scene"""
@@ -89,8 +97,10 @@ class GtkViewArea(gtk.DrawingArea):
         
     def redraw(self):
         """Request this widget is redrawn"""
-        #FIXME: Check this is valid
-        self.window.invalidate_rect(self.allocation, False)
+        # If the window is visible prepare it for redrawing
+        area = gtk.gdk.Rectangle(0, 0, self.allocation.width, self.allocation.height)
+        if self.window is not None:
+            self.window.invalidate_rect(area, False)
 
     def setRenderGL(self, renderGL):
         """Enable OpenGL rendering"""
@@ -214,17 +224,20 @@ class GtkView(glchess.ui.ViewController):
     """
     """
     # The UI this view belongs to
-    ui            = None
+    ui               = None
     
     # The title of this view
-    title         = None
+    title            = None
     
     # The widget to render the scene to
-    widget        = None
+    widget           = None
     
     # A Gtk+ tree model to store the move history
-    moveModel    = None
-    selectedMove = -1
+    moveModel        = None
+    selectedMove     = -1
+    
+    # Flag to show if requesting attention
+    requireAttention = False
     
     def __init__(self, ui, title, feedback, isActive = True):
         """Constructor for a view.
@@ -251,15 +264,34 @@ class GtkView(glchess.ui.ViewController):
     def render(self):
         """Extends glchess.ui.ViewController"""
         self.widget.redraw()
-        
-    def addMove(self, move):
+
+    def setAttention(self, requiresAttention):
         """Extends glchess.ui.ViewController"""
+        if self.requireAttention == requiresAttention:
+            return
+        self.requireAttention = requiresAttention
+        if requiresAttention:
+            self.ui._incAttentionCounter(1)
+        else:
+            self.ui._incAttentionCounter(-1)
+
+    def addMove(self, move):
+        """Extends glchess.ui.ViewController
+	Based on the current move, this method creates a string
+	describing the move, and displays it in the GUI.
+	"""
         # FIXME: Make a '@ui' player who watches for these itself?
         iter = self.moveModel.append()
         string = '%2i. ' % ((move.number - 1) / 2 + 1)
         if move.number % 2 == 0:
             string += '... '
         string += move.sanMove
+
+        if move.result is glchess.chess.board.MOVE_RESULT_OPPONENT_CHECK:
+            string += " - " + gettext.gettext('Check')
+        if move.result is glchess.chess.board.MOVE_RESULT_OPPONENT_CHECKMATE:
+            string += " - " + gettext.gettext('Checkmate, %s wins.' % (move.player.getName()))
+
         self.moveModel.set(iter, 0, move.number, 1, string)
         
         # If is the current view and tracking the game select this
@@ -268,6 +300,8 @@ class GtkView(glchess.ui.ViewController):
     
     def close(self):
         """Extends glchess.ui.ViewController"""
+        if self.requireAttention:
+            self.ui._incAttentionCounter(-1)
         self.ui._removeView(self)
     
     # Public methods
@@ -424,7 +458,7 @@ class AIView:
         """
         """
         self.window = window
-        self.__gui = loadGladeFile('ai.glade', 'ai_table', domain = 'glchess')
+        self.__gui = loadGladeFile('ai.glade', 'ai_table')
         self.__gui.get_widget('executable_label').set_text(executable)
         self.__gui.get_widget('game_label').set_text(description)
 
@@ -510,41 +544,46 @@ class GtkUI(glchess.ui.UI):
     """
     """
     # The Gtk+ GUI
-    _gui              = None
+    _gui               = None
     
     # The time stored for animation
-    __lastTime        = None
-    __animationTimer  = None
+    __lastTime         = None
+    __animationTimer   = None
+    
+    # Timers
+    timers             = None
 
     # The notebook containing games
-    notebook        = None
-    
+    notebook           = None
+
     # The Gtk+ list model of the available player types
-    __playerModel     = None
-    
+    __playerModel      = None
+
     # The about dialog open
-    __aboutDialog     = None
-    
+    __aboutDialog      = None
+
     # Dictionary of save game dialogs keyed by view
-    __saveGameDialogs = None
-    
-    __renderGL        = False
-    openGLInfoPrinted = False
+    __saveGameDialogs  = None
+
+    __renderGL         = False
+    openGLInfoPrinted  = False
 
     # TODO
-    __joinGameDialogs = None
-    __networkGames    = None
-    
-    __defaultWhiteAI  = None
-    __defaultBlackAI  = None
+    __joinGameDialogs  = None
+    __networkGames     = None
     
+    __defaultWhiteAI   = None
+    __defaultBlackAI   = None
+
+    __attentionCounter = 0
+
     def __init__(self):
         """Constructor for a GTK+ glChess GUI"""
         self.__networkGames = {}
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
         
-        self._gui = loadGladeFile('glchess.glade', domain = 'glchess')
+        self._gui = loadGladeFile('glchess.glade')
         self._gui.signal_autoconnect(self)
 
         # Make a notebook for the games
@@ -572,10 +611,10 @@ class GtkUI(glchess.ui.UI):
         combo.add_attribute(cell, 'text', 1)
 
         self.defaultViewController = self.notebook.setDefault(None)
-        
-        # Disable OpenGL support
-        if not hasattr(gtk, 'gtkgl'):
-            self._gui.get_widget('menu_view_3d').set_sensitive(False)
+
+        # Disble help support
+        if haveGnomeSupport:
+            self._gui.get_widget('menu_help').show()
         
         self.defaultViewController.widget.setRenderGL(self.__renderGL)
 
@@ -586,12 +625,28 @@ class GtkUI(glchess.ui.UI):
         glchess.config.watch('show_history', self.__applyConfig)
         glchess.config.watch('fullscreen', self.__applyConfig)
         glchess.config.watch('show_3d', self.__applyConfig)
+        glchess.config.watch('width', self.__applyConfig)
+        glchess.config.watch('height', self.__applyConfig)
 
     # Public methods
     
     def watchFileDescriptor(self, fd):
         """Extends ui.UI"""
         gobject.io_add_watch(fd, gobject.IO_IN, self.__readData)
+        
+    def addTimer(self, method, timeUsec):
+        """Extends ui.UI"""
+        timer = gobject.timeout_add(timeUsec / 1000, self.__timerExpired, method)
+        self.timers[method] = timer
+
+    def __timerExpired(self, method):
+        method()
+        return True
+
+    def removeTimer(self, method):
+        """Extends ui.UI"""
+        timer = self.timers.pop(method)
+        gobject.source_remove(timer)
 
     def __readData(self, fd, condition):
         return self.onReadFileDescriptor(fd)
@@ -634,10 +689,24 @@ class GtkUI(glchess.ui.UI):
         """Run the UI.
         
         This method will not return.
-        """
+        """        
         # Load configuration
-        for name in ['show_toolbar', 'show_history', 'fullscreen', 'show_3d']:
-            self.__applyConfig(name, glchess.config.get(name))
+        for name in ['show_toolbar', 'show_history', 'show_3d']:
+            try:
+                value = glchess.config.get(name)
+            except glchess.config.Error:
+                pass
+            else:
+                self.__applyConfig(name, value)
+        self.__resize()
+        
+        self._gui.get_widget('glchess_app').show()
+
+        # Apply the fullscreen flag after the window has been shown otherwise
+        # gtk.Window.unfullscreen() stops working if the window is set to fullscreen
+        # before being shown. I haven't been able to reproduce this in the simple
+        # case (GTK+ 2.10.6-0ubuntu3).
+        self.__applyConfig('fullscreen', glchess.config.get('fullscreen'))
         
         gtk.main()
         
@@ -662,7 +731,7 @@ class GtkUI(glchess.ui.UI):
         # Update the open dialogs
         for dialog in self.__joinGameDialogs:
             dialog.addNetworkGame(name, game)
-    
+
     def removeNetworkGame(self, game):
         """Extends glchess.ui.UI"""
         self.__networkGames.pop(game)
@@ -672,6 +741,22 @@ class GtkUI(glchess.ui.UI):
             dialog.removeNetworkGame(game)
 
     # Protected methods
+    
+    def _incAttentionCounter(self, offset):
+        """
+        """
+        self.__attentionCounter += offset
+        self.__updateAttention()
+        
+    def __updateAttention(self):
+        """
+        """
+        widget = self._gui.get_widget('glchess_app')
+        widget.set_urgency_hint(self.__attentionCounter != 0 and not widget.is_active())
+        
+    def _on_focus_changed(self, widget, data = None):
+        """Gtk+ callback"""
+        self.__updateAttention()
 
     def _saveView(self, view, path):
         """
@@ -693,9 +778,22 @@ class GtkUI(glchess.ui.UI):
 
     # Private methods
 
+    def __resize(self):
+        try:
+            width = glchess.config.get('width')
+            height = glchess.config.get('height')
+        except glchess.config.Error:
+            return
+        
+        self._gui.get_widget('glchess_app').resize(width, height)
+
     def __applyConfig(self, name, value):
         """
         """
+        if name == 'width' or name == 'height':
+            self.__resize()
+            return
+
         # Show/hide the toolbar
         if name == 'show_toolbar':
             toolbar = self.__getWidget('toolbar')
@@ -731,7 +829,11 @@ class GtkUI(glchess.ui.UI):
                 self._gui.get_widget('menu_fullscreen').show()
 
         # Enable/disable OpenGL rendering
-        elif name == 'show_3d':
+        elif name == 'show_3d':            
+            if value and not haveGLSupport:
+                self._gui.get_widget('3d_support_dialog').show()
+                glchess.config.set('show_3d', False)
+                value = False
             self.__renderGL = value
             menuItem = self.__getWidget('menu_view_3d')
             menuItem.set_active(self.__renderGL)
@@ -867,6 +969,14 @@ class GtkUI(glchess.ui.UI):
 
     def _on_view_changed(self, widget, page, pageNum, data = None):
         """Gtk+ callback"""
+        # Set the window title to the name of the game
+        view = self.notebook.getView()
+        title = gettext.gettext('Chess')
+        if view is not None:
+            title += " - %s" % view.title
+        self._gui.get_widget('glchess_app').set_title(title)
+        
+        # Set toolbar/menu buttons to state for this game
         self._updateViewButtons()
     
     def _updateViewButtons(self):
@@ -894,8 +1004,7 @@ class GtkUI(glchess.ui.UI):
 
     def _on_new_game_button_clicked(self, widget, data = None):
         """Gtk+ callback"""
-        
-        dialogs.GtkNewGameDialog(self, self.__playerModel, whiteAI = self.__defaultWhiteAI, blackAI = self.__defaultBlackAI)
+        dialogs.GtkNewGameDialog(self, self.__playerModel)
 
     def _on_join_game_button_clicked(self, widget, data = None):
         """Gtk+ callback"""
@@ -944,16 +1053,46 @@ class GtkUI(glchess.ui.UI):
     def _on_view_unfullscreen_clicked(self, widget, data = None):
         """Gtk+ callback"""
         glchess.config.set('fullscreen', False)
+        
+    def _on_3d_support_dialog_delete_event(self, widget, data = None):
+        """Gtk+ callback"""
+        # Stop the dialog from deleting itself
+        return True
+        
+    def _on_3d_support_dialog_response(self, widget, data = None):
+        """Gtk+ callback"""
+        if self.__aboutDialog is not None:
+            return
+        widget.hide()
+        return False
 
     def _on_about_clicked(self, widget, data = None):
         """Gtk+ callback"""
-        if self.__aboutDialog is None:
-            self.__aboutDialog = loadGladeFile('about.glade', domain = 'glchess')
-            self.__aboutDialog.signal_autoconnect(self)
+        if self.__aboutDialog is not None:
+            return
+        
+        dialog = self.__aboutDialog = gtk.AboutDialog()
+        dialog.set_name(APPNAME)
+        dialog.set_version(VERSION)
+        dialog.set_copyright(COPYRIGHT)
+        dialog.set_license(LICENSE[0] + '\n\n' + LICENSE[1] + '\n\n' +LICENSE[2])
+        dialog.set_wrap_license(True)
+        dialog.set_comments(DESCRIPTION)
+        dialog.set_authors(AUTHORS)
+        dialog.set_artists(ARTISTS)
+        string = ''
+        for t in TRANSLATORS:
+            string += t + '\n'
+        dialog.set_translator_credits(string[:-1])
+        dialog.set_website(WEBSITE)
+        dialog.set_website_label(WEBSITE_LABEL)
+        dialog.set_logo_icon_name('glchess')
+        dialog.connect('response', self._on_glchess_about_dialog_close)
+        dialog.show()
         
     def _on_glchess_about_dialog_close(self, widget, data = None):
         """Gtk+ callback"""
-        self.__aboutDialog.get_widget('glchess_about_dialog').destroy()
+        self.__aboutDialog.destroy()
         self.__aboutDialog = None
         return False
         
@@ -982,6 +1121,11 @@ class GtkUI(glchess.ui.UI):
         """Gtk+ callback"""
         # Leave it to use to hide the dialog
         return True
+    
+    def _on_resize(self, widget, event, data = None):
+        """Gtk+ callback"""
+        glchess.config.set('width', event.width)
+        glchess.config.set('height', event.height)
 
     def _on_quit(self, widget, data = None):
         """Gtk+ callback"""
